==> z-job-support-0.2.2-sources.jar: cc/zuv/job/support/impl/quartz/executor/QuartzEventExecutor.java <==
/*
 * Id$: zuv-cloud:z-job-support:cc.zuv.job.support.impl.quartz.executor.QuartzEventExecutor:20190104164249
 *
 * QuartzEventExecutor.java
 * Copyright (c) 2002-2020 Luther Inc.
 * http://zuv.cc
 * All rights reserved.
 */

package cc.zuv.job.support.impl.quartz.executor;

import cc.zuv.ios.exception.HttpClientException;
import cc.zuv.ios.httpconn.IHttpRes;
import cc.zuv.ios.httpconn.httpok.OkHttpConn;
import cc.zuv.job.support.IJobCode;
import cc.zuv.job.support.core.IExecContext;
import cc.zuv.job.support.core.IExecutor;
import cc.zuv.job.support.data.dao.JobsFireEntityRepository;
import cc.zuv.job.support.data.domain.JobsFireEntity;
import cc.zuv.job.support.data.enums.TaskStatus;
import cc.zuv.lang.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 事件触发操作,允许并发
 *
 * @author          Kama Luther
 * @version         0.1
 * @since           0.1
 * @create.date     2018-12-24 17:26
 * @modify.date     2018-12-24 17:26
 */
@Slf4j
@PersistJobDataAfterExecution
public abstract class QuartzEventExecutor implements IExecutor, IJobCode, Job
{

    //-----------------------------------------------------------------------------------------

    @Autowired
    protected JobsFireEntityRepository jobsFireEntityRepository;

    //-----------------------------------------------------------------------------------------

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException
    {
        //
        JobDetail detail = context.getJobDetail();
        if(detail.getJobDataMap()==null
            || !detail.getJobDataMap().containsKey(JOBS_MAPDATA_NAME_FIREID)) return;

        //
        String key = detail.getKey().getName() + " @ " + detail.getKey().getGroup();
        String desc = detail.getDescription();
        Long fireid = detail.getJobDataMap().getLong(JOBS_MAPDATA_NAME_FIREID);
        String firereqcode = detail.getJobDataMap().getString(JOBS_MAPDATA_NAME_FIREREQCODE);
        String firereqdata = detail.getJobDataMap().getString(JOBS_MAPDATA_NAME_FIREREQDATA);
        log.debug("{} ({}) : {} {} {}", key, desc, fireid, firereqcode, firereqdata);

        //
        JobsFireEntity entity = jobsFireEntityRepository.findOne(fireid);
        if(entity==null)
        {
            log.error("JobsFireEntity(id={}) not found.", fireid);
            return;
        }
        entity.setStatus(TaskStatus.RUNNING);
        entity.setBeginTime(System.currentTimeMillis());
        entity = jobsFireEntityRepository.saveAndFlush(entity);

        try
        {
            //修改resdata, endtime, status
            execute(new QuartzExecContext(detail));
        }
        catch (Exception e)
        {
            log.error("execute error {}", e.getMessage());

            //
            entity.setStatus(TaskStatus.ERROR);
            entity.setEndTime(System.currentTimeMillis());
            entity.setMessage(e.getMessage());
            jobsFireEntityRepository.saveAndFlush(entity);

//            // 策略1: 修复错误并立即重新启动
//            JobExecutionException e2 = new JobExecutionException(e);
//            detail.getJobDataMap().put("fixed", true);
//            e2.setRefireImmediately(true);

            // 策略2: 自动取消与此作业相关联的所有触发器, 以使其不再运行
            JobExecutionException e2 = new JobExecutionException(e);
            e2.setUnscheduleFiringTrigger(true);
//            e2.setUnscheduleAllTriggers(true);
            throw e2;
        }
        finally
        {
            //调用回调
            entity = jobsFireEntityRepository.findOne(fireid); //再查询一次避免乐观锁
            if(entity!=null && (entity.getCbexeced()==null || !entity.getCbexeced())
                && StringUtils.NotEmpty(entity.getCbapiurl()) )
            {
                String url = entity.getCbapiurl();
                url += url.contains("?")?"&":"?";
                url += "fireid=" + entity.getId();
                String result;
                try
                {
                    IHttpRes response = OkHttpConn.conn(url).timeout(60, 120, 120).get();
                    result = response.success() ? response.string() : response.status() + " : " + response.message();
                }
                catch (HttpClientException e)
                {
                    log.error("Callback failure : " + e.getMessage(), e);
                    result = e.getMessage();
                }
                entity.setCbresult(result);
                entity.setCbexeced(true);
                jobsFireEntityRepository.saveAndFlush(entity);
            }
        }

    }


    public abstract void execute(IExecContext context);

    //-----------------------------------------------------------------------------------------


}

==> z-job-support-0.2.5-sources.jar: cc/zuv/job/support/impl/quartz/executor/QuartzEventExecutor.java <==
/*
 * Id$: zuv-cloud:z-job-support:cc.zuv.job.support.impl.quartz.executor.QuartzEventExecutor:20190104164249
 *
 * QuartzEventExecutor.java
 * Copyright (c) 2002-2020 Luther Inc.
 * http://zuv.cc
 * All rights reserved.
 */

package cc.zuv.job.support.impl.quartz.executor;

import cc.zuv.ios.exception.HttpClientException;
import cc.zuv.ios.httpconn.IHttpRes;
import cc.zuv.ios.httpconn.httpok.OkHttpConn;
import cc.zuv.job.support.IJobCode;
import cc.zuv.job.support.core.IExecContext;
import cc.zuv.job.support.core.IExecutor;
import cc.zuv.job.support.data.dao.JobsFireEntityRepository;
import cc.zuv.job.support.data.domain.JobsFireEntity;
import cc.zuv.job.support.data.enums.TaskStatus;
import cc.zuv.lang.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 事件触发操作,允许并发
 *
 * @author          Kama Luther
 * @version         0.1
 * @since           0.1
 * @create.date     2018-12-24 17:26
 * @modify.date     2018-12-24 17:26
 */
@Slf4j
@PersistJobDataAfterExecution
public abstract class QuartzEventExecutor implements IExecutor, IJobCode, Job
{

    //-----------------------------------------------------------------------------------------

    @Autowired
    protected JobsFireEntityRepository jobsFireEntityRepository;

    //-----------------------------------------------------------------------------------------

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException
    {
        //
        JobDetail detail = context.getJobDetail();
        if(detail.getJobDataMap()==null
            || !detail.getJobDataMap().containsKey(JOBS_MAPDATA_NAME_FIREID)) return;

        //
        String key = detail.getKey().getName() + " @ " + detail.getKey().getGroup();
        String desc = detail.getDescription();
        Long fireid = detail.getJobDataMap().getLong(JOBS_MAPDATA_NAME_FIREID);
        log.debug("{} ({}) : {}", key, desc, fireid);

        //
        JobsFireEntity entity = jobsFireEntityRepository.findOne(fireid);
        if(entity==null)
        {
            log.error("JobsFireEntity(id={}) not found.", fireid);
            return;
        }
        entity.setStatus(TaskStatus.RUNNING);
        entity.setBeginTime(System.currentTimeMillis());
        entity = jobsFireEntityRepository.saveAndFlush(entity);

        try
        {
            //修改resdata, endtime, status
            execute(new QuartzExecContext(detail));
        }
        catch (Exception e)
        {
            log.error("execute error {}", e.getMessage());

            //
            entity.setStatus(TaskStatus.ERROR);
            entity.setEndTime(System.currentTimeMillis());
            entity.setMessage(e.getMessage());
            jobsFireEntityRepository.saveAndFlush(entity);

//            // 策略1: 修复错误并立即重新启动
//            JobExecutionException e2 = new JobExecutionException(e);
//            detail.getJobDataMap().put("fixed", true);
//            e2.setRefireImmediately(true);

            // 策略2: 自动取消与此作业相关联的所有触发器, 以使其不再运行
            JobExecutionException e2 = new JobExecutionException(e);
            e2.setUnscheduleFiringTrigger(true);
//            e2.setUnscheduleAllTriggers(true);
            throw e2;
        }
        finally
        {
            //调用回调
            entity = jobsFireEntityRepository.findOne(fireid); //再查询一次避免乐观锁
            if(entity!=null && (entity.getCbexeced()==null || !entity.getCbexeced())
                && StringUtils.NotEmpty(entity.getCbapiurl()) )
            {
                String url = entity.getCbapiurl();
                url += url.contains("?")?"&":"?";
                url += "fireid=" + entity.getId();
                url += "&reqcode=" + entity.getReqcode();
                String result;
                try
                {
                    IHttpRes response = OkHttpConn.conn(url).ssl(true).timeout(60, 120, 120).get();
                    result = response.success() ? response.string() : response.status() + " : " + response.message();
                }
                catch (HttpClientException e)
                {
                    log.error("Callback failure : " + e.getMessage(), e);
                    result = e.getMessage();
                }
                entity.setCbresult(result);
                entity.setCbexeced(true);
                jobsFireEntityRepository.saveAndFlush(entity);
            }
        }

    }

    public abstract void execute(IExecContext context);

    //-----------------------------------------------------------------------------------------

    protected void begin()
    {
        log.info("\n");
        log.info("-----------------------------------");
        log.info("BEGIN");
    }

    protected void end()
    {
        log.info("END");
        log.info("-----------------------------------");
    }

    //-----------------------------------------------------------------------------------------

}
